"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeModelFile = exports.generateModelFiles = exports.modelExtendTemplatePath = exports.modelTemplatePath = exports.ENCODING = void 0;
const fs_1 = require("fs");
const handlebars_1 = require("handlebars");
const path_1 = require("path");
const mkdirp_1 = __importDefault(require("mkdirp"));
const utils_1 = require("../utils");
const TEMPLATE_FOLDER = path_1.resolve(__dirname, "..", "templates");
exports.ENCODING = "utf8";
exports.modelTemplatePath = `${TEMPLATE_FOLDER}/model.template.hbs`;
exports.modelExtendTemplatePath = `${TEMPLATE_FOLDER}/model-extend.template.hbs`;
function generateModelFiles(service, definitions) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!service.config.model || !service.config.modelDir || !definitions) {
            return;
        }
        const modelClassContent = getDefinitionItems(definitions)
            .map(({ className, properties }) => generateModelContent(className, properties))
            .join("\r\n");
        const modelImportContent = getImportContent();
        const modelFileContent = [modelImportContent, modelClassContent].join("\r\n");
        yield writeModelFile(service, modelFileContent);
    });
}
exports.generateModelFiles = generateModelFiles;
function getImportContent() {
    return [
        `import { Type } from 'class-transformer'`,
        `import { Model } from '@gopowerteam/http-request'`,
        "\r\n",
    ].join("\r\n");
}
function getDefinitionItems(definitions) {
    return Object.entries(definitions || {})
        .map(([className, { type, properties }]) => ({
        className,
        type,
        properties,
    }))
        .filter(({ className, type }) => !className.startsWith("Map«") &&
        !className.startsWith("Page«") &&
        !className.startsWith("Iterable«") &&
        !className.startsWith("Pageable") &&
        !className.startsWith("Serializable") &&
        type == "object");
}
function generateModelContent(className, properties) {
    const templateSource = fs_1.readFileSync(exports.modelTemplatePath, exports.ENCODING);
    const template = handlebars_1.compile(templateSource, { noEscape: true });
    return template({
        className: className,
        properties: getformatProperty(properties),
    });
}
function getformatProperty(properties) {
    return Object.entries(properties).map(([propertyName, propertyConfig]) => {
        return {
            propertyName,
            propertyType: getPropertyType(propertyConfig),
            originalRef: getOriginalRef(propertyConfig),
            description: propertyConfig.description,
        };
    });
}
function getOriginalRef(propertyConfig) {
    var _a;
    const ref = (propertyConfig === null || propertyConfig === void 0 ? void 0 : propertyConfig.originalRef) || ((_a = propertyConfig === null || propertyConfig === void 0 ? void 0 : propertyConfig.items) === null || _a === void 0 ? void 0 : _a.originalRef);
    if (ref && ref.startsWith("Map«")) {
        return;
    }
    return ref;
}
function getPropertyType(config) {
    switch (true) {
        case !!config.originalRef:
            return config.originalRef
                .replace(/^Map«/g, "Record<")
                .replace(/»$/g, ">");
        case config.type === "integer":
            return "number";
        case config.type === "array":
            return `${getPropertyType(config.items)}[]`;
    }
    return config.type;
}
/**
 * 生成控制器文件
 * @param service
 * @param param1
 * @param content
 */
function writeModelFile(service, content) {
    return __awaiter(this, void 0, void 0, function* () {
        const modelDirectionPath = service.config.modelDir;
        const name = service.key ? `${service.key}.model.ts` : "index.ts";
        const path = path_1.resolve(modelDirectionPath, name);
        yield mkdirp_1.default.sync(path_1.dirname(path));
        yield fs_1.writeFileSync(path, content, exports.ENCODING);
        utils_1.log("Model File Generate", `${path}`);
    });
}
exports.writeModelFile = writeModelFile;
