"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateModelFiles = exports.modelExtendTemplatePath = exports.modelTemplatePath = exports.ENCODING = void 0;
const utils_1 = require("../utils");
const { readFileSync, writeFileSync } = require("fs");
const { compile } = require("handlebars");
const { dirname, resolve } = require("path");
const mkdirp = require("mkdirp");
const TEMPLATE_FOLDER = resolve(__dirname, "..", "templates");
exports.ENCODING = "utf8";
exports.modelTemplatePath = `${TEMPLATE_FOLDER}/model.template.hbs`;
exports.modelExtendTemplatePath = `${TEMPLATE_FOLDER}/model-extend.template.hbs`;
function generateModelFiles(service, components) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!service.config.model || !service.config.modelDir) {
            return;
        }
        const modelClassContent = getDefinitionItems(components.schemas)
            .map(({ className, properties }) => generateModelContent(className, properties))
            .join("\r\n");
        const modelImportContent = getImportContent();
        const modelFileContent = [modelImportContent, modelClassContent].join("\r\n");
        yield writeModelFile(service, modelFileContent);
    });
}
exports.generateModelFiles = generateModelFiles;
function getImportContent() {
    return [
        `import { Type } from 'class-transformer'`,
        `import { Model } from '@gopowerteam/http-request'`,
        "\r\n",
    ].join("\r\n");
}
function getDefinitionItems(definitions) {
    return Object.entries(definitions)
        .map(([className, { type, properties }]) => ({
        className,
        type,
        properties,
    }))
        .filter(({ className, type }) => !className.startsWith("Map") &&
        !className.startsWith("Page") &&
        !className.startsWith("Pageable") &&
        !className.startsWith("Serializable") &&
        !className.startsWith("Predicate") &&
        !className.startsWith("Sort") &&
        type == "object");
}
function generateModelContent(className, properties) {
    const templateSource = readFileSync(exports.modelTemplatePath, exports.ENCODING);
    const template = compile(templateSource);
    return template({
        className: className,
        properties: getformatProperty(properties),
    });
}
function getformatProperty(properties) {
    return Object.entries(properties).map(([propertyName, propertyConfig]) => {
        return {
            propertyName,
            propertyType: getPropertyType(propertyConfig),
            originalRef: getOriginalRef(propertyConfig),
            description: propertyConfig.description,
        };
    });
}
function getOriginalRef(propertyConfig) {
    var _a, _b;
    return (_b = ((propertyConfig === null || propertyConfig === void 0 ? void 0 : propertyConfig.$ref) || ((_a = propertyConfig === null || propertyConfig === void 0 ? void 0 : propertyConfig.items) === null || _a === void 0 ? void 0 : _a.$ref))) === null || _b === void 0 ? void 0 : _b.replace("#/components/schemas/", "");
}
function getPropertyType(config) {
    var _a, _b;
    const ref = config.$ref || ((_b = (_a = config["allOf"]) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.$ref);
    switch (true) {
        case !!ref:
            return ref.replace("#/components/schemas/", "");
        case config.type === "integer":
            return "number";
        case config.type === "array":
            return `${getPropertyType(config.items)}[]`;
    }
    return config.type;
}
/**
 * 生成控制器文件
 * @param service
 * @param param1
 * @param content
 */
function writeModelFile(service, content) {
    return __awaiter(this, void 0, void 0, function* () {
        const modelDirectionPath = service.config.modelDir;
        const name = service.key ? `${service.key}.model.ts` : "index.ts";
        const path = resolve(modelDirectionPath, name);
        yield mkdirp.sync(dirname(path));
        yield writeFileSync(path, content, exports.ENCODING);
        utils_1.log("Model File Generate", `${path}`);
    });
}
