"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeServiceFile = exports.generateServiceFile = exports.generateServiceFiles = exports.serviceTemplatePath = exports.ENCODING = void 0;
const handlebars_1 = require("handlebars");
const fs_1 = require("fs");
const path_1 = require("path");
const utils_1 = require("../utils");
const mkdirp_1 = __importDefault(require("mkdirp"));
const TEMPLATE_FOLDER = path_1.resolve(__dirname, "..", "templates");
exports.ENCODING = "utf8";
exports.serviceTemplatePath = `${TEMPLATE_FOLDER}/service.template.hbs`;
// export const serviceDirectionPath = loadConfig().serviceDir
function generateServiceFiles(service, controllers) {
    controllers.forEach((controller) => generateServiceFile(service, controller));
}
exports.generateServiceFiles = generateServiceFiles;
function generateServiceFile(service, controller) {
    let templateSource = fs_1.readFileSync(exports.serviceTemplatePath, exports.ENCODING);
    let template = handlebars_1.compile(templateSource);
    const schemas = [
        ...new Set(controller.actions
            .map((action) => action.schema && action.schema.replace("[]", ""))
            .filter((x) => !!x)),
    ];
    let serviceFileContent = template(Object.assign(controller, {
        service: service.key,
        controllerDir: [service.config.controllerAlias, service.key]
            .filter((x) => x)
            .join("/"),
        modelDir: [service.config.modelAlias].filter((x) => x).join("/"),
        schemas: schemas.length ? schemas : undefined,
    }));
    writeServiceFile(service.key, controller, serviceFileContent, service.config).then((filename) => {
        utils_1.log(`Service File Generate`, filename);
    });
}
exports.generateServiceFile = generateServiceFile;
function writeServiceFile(service, { controller, filename }, content, config) {
    return __awaiter(this, void 0, void 0, function* () {
        const serviceDirectionPath = config.serviceDir;
        const path = path_1.resolve(serviceDirectionPath, service, `${filename}.service.ts`);
        yield mkdirp_1.default.sync(path_1.dirname(path));
        yield fs_1.writeFileSync(path, content, exports.ENCODING);
        return path;
    });
}
exports.writeServiceFile = writeServiceFile;
