"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.info = exports.log = exports.loadConfig = void 0;
const chalk_1 = __importDefault(require("chalk"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const CONFIG_FILE = [
    "http-request-cli.config.cjs",
    "http-request-cli.config.js",
];
let CONFIG_DATA = null;
/**
 * 加载配置信息
 * @returns
 */
function loadConfig() {
    if (CONFIG_DATA)
        return CONFIG_DATA;
    const filePath = CONFIG_FILE.find((file) => fs_1.default.existsSync(file));
    if (!filePath) {
        throw Error("未找到配置文件");
    }
    else {
        console.info(`发现配置文件: ${filePath}`);
    }
    const configJson = require(path_1.default.resolve("./", filePath));
    const getConfig = (config) => {
        var _a, _b, _c;
        return (Object.assign(Object.assign({}, config), { controllerAlias: config.controllerDir.alias, controllerDir: path_1.default.resolve(config.controllerDir.path), serviceAlias: config.serviceDir.alias, serviceDir: path_1.default.resolve(config.serviceDir.path), modelAlias: (_a = config.modelDir) === null || _a === void 0 ? void 0 : _a.alias, modelDir: ((_b = config.modelDir) === null || _b === void 0 ? void 0 : _b.path) && path_1.default.resolve((_c = config.modelDir) === null || _c === void 0 ? void 0 : _c.path) }));
    };
    if (Array.isArray(configJson)) {
        CONFIG_DATA = configJson.map(getConfig);
    }
    else {
        CONFIG_DATA = getConfig(configJson);
    }
    return CONFIG_DATA;
}
exports.loadConfig = loadConfig;
function log(type, message) {
    console.log(chalk_1.default `{blue.bold [${type}]} {gray ${message}}`);
}
exports.log = log;
function info(type, message = "") {
    console.log(chalk_1.default `{red.bold ${type}} {green.bold ${message}}`);
}
exports.info = info;
